<?php
/**
 * Created by PhpStorm.
 * User: rafaelporto
 * Date: 04/04/2016
 * Time: 10:46
 */

namespace Controller;

use \Service\ACL\AccessProfile as AccessProfileService;
use Util\Utilities;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;


class AccessProfile extends Crud{

    const IS_NEW = "__is_new";

    protected $dataSourceName = "accessProfile";
    protected $dataSourceNameIsTeknisa = "accessProfileIsTeknisa";
    protected $environment ;

    /** @var AccessProfileService */
    protected $accessProfileService;

    public function __construct(Manager $manager, AccessProfileService $accessProfileService)
    {
        $this->accessProfileService = $accessProfileService;
        $this->environment = Utilities::getEnvironment();
        parent::__construct($manager);
    }

    public function findAccessProfile(DTO\Request\Filter $request, DTO\Response $response) {
        if(Utilities::currentUserIsTeknisa()){
            $this->dataSourceName = $this->dataSourceNameIsTeknisa;
        }else{
            $request->getFilterCriteria()->addCondition('CDCLIENTE', $this->environment->getCurrentCdCliente());
        }
        $request->getFilterCriteria()->addCondition('NRORG', $this->environment->getCurrentOrganizationId());
        parent::find($request, $response);
    }

    public function findCpPerfacessoper(DTO\Request\Filter $request, DTO\Response $response) {
        if(Utilities::currentUserIsTeknisa()){
            $this->dataSourceName = "perfAcessParcOperador";
        }else{
            $this->dataSourceName = "perfAcessParc";
        }
        $request->getFilterCriteria()->addCondition('NRORG', $this->environment->getCurrentOrganizationId());
        parent::find($request, $response);
    }

    private function getDefaultRuleDataSet(){
        $defaultRuleRow = array(
            'defaultRule' => 'A'
        );
        return new DataSet('zhu_access_control_default_rule', $defaultRuleRow);
    }

    public function findAccessControlRule(DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $groupId = Utilities::getValueInFilterDataSourceByName($filters,'GROUP_ID');
        if($groupId && !empty($groupId[0]) && is_array($groupId[0])){
            $groupId = \Zeedhi\DevOps\Util\Utilities::getArrayFromArrayAssociative($groupId);
        }
        $accessControlRule = $this->accessProfileService->findAccessControlRule($groupId);
        $accessControlRuleDataSet = new DataSet("zhu_access_control_rule_login",$accessControlRule);
        $response->addDataSet($accessControlRuleDataSet);
        $response->addDataSet($this->getDefaultRuleDataSet());
    }

    public function findAccessControlOperador(DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $nrperfilacesso = Utilities::getValueInFilterDataSourceByName($filters,'NRPERFILACESSO');
        $accessControlRule = $this->accessProfileService->findAccessControlOperador($nrperfilacesso);
        $accessControlRuleDataSet = new DataSet("zhu_access_control_rule_login",$accessControlRule);
        $response->addDataSet($accessControlRuleDataSet);
    }

    public function getAccessProfile(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $accessProfile = $this->accessProfileService->getAccessProfile();
            $response->addDataSet(new DataSet('system', $accessProfile));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response \Message::TYPE_ERROR));
            $response->setError(new DTO\Response\Error($e->getMessage(), DTO\Response \Message::TYPE_ERROR));
        }
    }

    public function findAclZhuAclElementRules(DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $groupId = Utilities::getValueInFilterDataSourceByName($filters,'GROUP_ID');
        $accessControlRule = $this->accessProfileService->findAclZhuAclElementRules($groupId);
        $accessControlRuleDataSet = new DataSet("acl_zhu_acl_element_rules",$accessControlRule);
        $response->addDataSet($accessControlRuleDataSet);
    }

    public function saveAccessProfile(DTO\Request\DataSet $request, DTO\Response $response) {

        $row = $request->getDataSet()->getRows()[0];

        $nmperfilacesso = $row['NMPERFILACESSO'];
        $nrperfilacesso = empty($row['NRPERFILACESSO'])?null:$row['NRPERFILACESSO'];
        $idativo = $row['IDATIVO'];
        $idperfilacessoadm = $row['IDPERFILACESSOADM'];
        $nrperfilacesso = $this->accessProfileService->saveAccessProfile($nrperfilacesso,$nmperfilacesso,$idativo,$idperfilacessoadm);

        $defaultValuesDataSet = new DataSet("accessProfile", array(array('NRPERFILACESSO' => $nrperfilacesso)));
        $response->addDataSet($defaultValuesDataSet);
    }

    public function saveCpPerfacessoper(DTO\Request\DataSet $request, DTO\Response $response) {

        $row = $request->getDataSet()->getRows()[0];
        $cdoperadors = $row['CDOPERADOR'];
        $nrperfilacesso = $row['NRPERFILACESSO'];
        $this->accessProfileService->addPerfilInOperador($cdoperadors, $nrperfilacesso);
    }

    public function deleteCpPerfacessoper(DTO\Request\DataSet $request, DTO\Response $response) {
        $cdoperadors = [];
        $nrperfilacesso = null;
        foreach($request->getDataSet()->getRows() as $row) {
            $cdoperadors[] = $row['CDOPERADOR'];
            $nrperfilacesso = $row['NRPERFILACESSO'];

        }
        $this->accessProfileService->deletePerfilInOperador($cdoperadors, $nrperfilacesso);
    }

    public function saveAclZhuAclElementRules(DTO\Request\DataSet $request, DTO\Response $response) {

        $ids = [];
        $ids['updated'] = [];
        $ruleContainer = null;
        foreach($request->getDataSet()->getRows() as $row) {
            $uniqueId = $row['UNIQUE_ID'];
            $name = $row['NAME'];
            $label = $row['LABEL'];
            $type = $row['TYPE'];
            $rule = $row['RULE'];
            $projectId = $row['PROJECT_ID'];
            $groupId = $row['GROUP_ID'];
            $ruleId = empty($row['RULE_ID']) ? null : $row['RULE_ID'];
            $containerUniqueId = empty($row['CONTAINER_ID']) ? null : $row['CONTAINER_ID'];
            $containerName = empty($row['CONTAINER_NAME']) ? null : $row['CONTAINER_NAME'];
            $containerLabel = empty($row['CONTAINER_LABEL']) ? null : $row['CONTAINER_LABEL'];
            $containerRuleId = empty($row['CONTAINER_RULE_ID']) ? null : $row['CONTAINER_RULE_ID'];
            if(!$ruleContainer){
                $ruleContainer = $rule;
            }
            $ids['updated'][$uniqueId] = $this->accessProfileService->saveAclZhuAclElementRules($ruleContainer,$type,$uniqueId,$projectId,$name,$label,
                $groupId,$ruleId,$containerUniqueId,$containerName,$containerLabel,$containerRuleId);
        }

        $defaultValuesDataSet = new DataSet("aclZhuAccessControlRule", $ids);
        $response->addDataSet($defaultValuesDataSet);
    }



    public function deleteAccessProfile(DTO\Request\DataSet $request, DTO\Response $response) {
        $profilesToDelete = array();
        foreach ($request->getDataSet()->getRows() as &$row) {
            $profilesToDelete[] = $row['NRPERFILACESSO'];
        }
        $this->accessProfileService->deleteDependencies($profilesToDelete);
        parent::delete($request, $response);
    }

    public function deleteRowAccessProfile(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $profileToDelete = array($row[AccessProfileService::NRPERFILACESSO]);
        $this->accessProfileService->deleteDependencies($profileToDelete);
        $dataSetToDelete = new DataSet($this->dataSourceName, array($row));
        $deletedRows = $this->dataSourceManager->delete($dataSetToDelete);
        $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        $nrDeletedRows = count($deletedRows);
        $response->addNotification(new DTO\Response\Notification("{$nrDeletedRows} row(s) deleted.", DTO\Response\Notification::TYPE_SUCCESS));
    }

}