<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Teknisa\Model\Teknisa\Entities;
use Teknisa\Model\Corevision\Entities\Bkgthread;
use Teknisa\Model\Corevision\Entities\Bkgthreadparam;
use Util\Utilities;
use Util\Messages;
use Service\Queue\DownloadVersion\Producer;
use Teknisa\Libs\Util\Masks\PictureProduto;

class DownloadVersion
{

    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var \Util\Utilities $environment */
    protected $environment;
    /** @var \Teknisa\Model\Teknisa\Repositories\Patchverprod $patchverprodRepository */
    protected $patchverprodRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\Solipatch $solipatchRepository */
    protected $solipatchRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\Soliserv $soliservRepository */
    protected $soliservRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Cliente $clienteRepository */
    protected $clienteRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Produto $produtoRepository */
    protected $produtoRepository;
    /** @var \Teknisa\Model\Corevision\Repositories\Bkgthread $bkgthreadRepository */
    protected $bkgthreadRepository;
    /** @var \Teknisa\Model\Corevision\Repositories\Bkgthreadparam  $bkgthreadparamRepository */
    protected $bkgthreadparamRepository;
    /** @var \Teknisa\Model\Controleproducao\Repositories\Autgrupop $autgrupopRepository */
    protected $autgrupopRepository;
    /** @var \Factory\Bkgthread*/
    protected $bkgthreadFactory;
    /** @var \Factory\Bkgthreadparam*/
    protected $bkgthreadparamFactory;
    /** @var  \Service\Teknisa\NewCode $newCodeService */
    protected $newCodeService;
    
    protected $downloadVersionProducer;
    
    const DEFAULT_NOT_CURRENT_CUSTOMER = 'T';
    const SOAP_RESPONSE_SUCCESS        = 'OK';
    const SOAP_INITIAL_RESPONSE_ERROR  = 'ERRO_';

    private $ALL_REQUESTS = array(
        'GENERATE_PATCH' => array(
            'RDM_METHOD' => 'PCH.rdmPatch.GeraPatch',
            'DESCRIPTION' => array(
                'REQUEST' => 'Atualização Completa',
                'GENERATION' => 'Geração de Patch. Cliente: {{customerName}}'
            )
        ),
        'GENERATE_FIX' => array(
            'RDM_METHOD' => 'PCH.rdmPatch.GeraFix',
            'DESCRIPTION' => array(
                'REQUEST' => 'Fix por Solicitação',
                'GENERATION' => 'Geração de Fix (Solic. {{requestNumber}}). Cliente: {{customerName}}'
            )
        ),
        'GENERATE_LANGUAGE_PATCH' => array(
            'RDM_METHOD' => 'PCH.rdmPatch.GeraPatchIdi',
            'DESCRIPTION' => array(
                'REQUEST' => 'Patch de Idioma',
                'GENERATION' => 'Geração de Patch (Idioma). Cliente: {{customerName}}'
            )
        ),
        'GENERATE_SQL_PATCH' => array(
            'RDM_METHOD' => 'PCH.rdmPatch.GeraPatchSql',
            'DESCRIPTION' => array(
                'REQUEST' => 'Patch de SQL',
                'GENERATION' => 'Geração de Patch (Arquivos BD). Cliente: {{customerName}}'
            )
        )
    );
    
    const DESCRIPTION_GENERATE_PATCH          = 'Geração de Patch. Cliente: {{customerName}}';
    const DESCRIPTION_GENERATE_FIX            = 'Geração de Fix (Solic. {{requestNumber}}). Cliente: {{customerName}}';
    const DESCRIPTION_GENERATE_LANGUAGE_PATCH = 'Geração de Patch (Idioma). Cliente: {{customerName}}';
    const DESCRIPTION_GENERATE_SQL_PATCH      = 'Geração de Patch (Arquivos BD). Cliente: {{customerName}}';
    
    const DESCRIPTION_FULL_UPDATE             = 'Atualização Completa';
    const DESCRIPTION_FIX_BY_REQUEST          = 'Fix por Solicitação';
    const DESCRIPTION_LANGUAGE_PATCH          = 'Patch de Idioma';
    const DESCRIPTION_SQL_PATCH               = 'Patch de SQL';

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, Producer $downloadVersionProducer, 
                                \Factory\Bkgthread $bkgthreadFactory, \Factory\Bkgthreadparam $bkgthreadparamFactory, \Service\Teknisa\NewCode $newCodeService) {
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->downloadVersionProducer  = $downloadVersionProducer;
        $this->bkgthreadFactory         = $bkgthreadFactory;
        $this->bkgthreadparamFactory    = $bkgthreadparamFactory;
        $this->clienteRepository        = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Cliente');
        $this->produtoRepository        = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Produto');
        $this->patchverprodRepository   = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Patchverprod');
        $this->solipatchRepository      = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Solipatchpro');
        $this->soliservRepository       = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Soliserv');
        $this->bkgthreadRepository      = $this->entityManager->getRepository('\Teknisa\Model\Corevision\Entities\Bkgthread');
        $this->bkgthreadparamRepository = $this->entityManager->getRepository('\Teknisa\Model\Corevision\Entities\Bkgthreadparam');
        $this->autgrupopRepository      = $this->entityManager->getRepository('\Teknisa\Model\Controleproducao\Entities\Autgrupop');
        $this->newCodeService           = $newCodeService;
    }
    
    private function updateClientDb($clientId, $dbId)
    {
        try {
            $client = $this->clienteRepository->findOneBy(array('cdcliente' => $clientId));
            if ($client){
                $client->setNmbdcliente($dbId);
            } else {
                throw new CustomException('ERR_NOT_FOUND_CUSTOMER');
            }
        } catch (\Exception $ex){
            throw new \Exception($ex->getMessage());
        }
    }
    
    public function getAuthorizationByCurrentUser()
    {
        $userId = $this->environment->getCurrentUserId();
        $authorizedOperator = $this->autgrupopRepository->getAuthorizedOperator($userId);
        return empty($authorizedOperator) ? false : true;
    }
    
    public function getDescriptionByRequest($requestId)
    {
        $descriptionInRTF = $this->patchverprodRepository->getDescriptionByRequest($requestId);
        if (!empty($descriptionInRTF) && substr($descriptionInRTF,0,5) == '{\rtf') {
            $descriptionInTextPlain = Utilities::rtf2textPlain($descriptionInRTF);
        }
        return array('REQUEST_DESCRIPTION' => $descriptionInTextPlain);
    }
    
    public function getRequestsAnswered($isCurrentClient, $initialPatch, $endPatch, $filters)
    {
        
        $clientId     = $isCurrentClient ? $this->environment->getCurrentCdCliente() : self::DEFAULT_NOT_CURRENT_CUSTOMER;
        $initialPatch = $this->validateAndFormatVersion($initialPatch);
        $endPatch     = $this->validateAndFormatVersion($endPatch);
        $conditions   = Utilities::createQueryFiltered($filters);
        return $this->patchverprodRepository->getRequestsAnswered($initialPatch, $endPatch, $clientId, $conditions);
    }
    
    public function getSolicitationAudit($isCurrentClient, $initialPatch, $endPatch, $filters)
    {
        $initialPatch = $this->validateAndFormatVersion($initialPatch);
        $endPatch     = $this->validateAndFormatVersion($endPatch);
        $conditions   = Utilities::createQueryFiltered($filters);
        return $this->patchverprodRepository->getSolicitationAudit($initialPatch, $endPatch, $isCurrentClient, $this->environment->getCurrentCdCliente());
    }
    
    
    
    public function getProductsAvailablesByVersion($version, $versionPatch, $languageId)
    {
        $cdsProdDisp    = array();
        $version        = $this->validateAndFormatVersion($version);
        $orgId          = $this->environment->getCurrentOrganizationId();
        $clientId       = $this->environment->getCurrentCdCliente();
        $mainVersion    = substr($version, 0,1);
        $releaseVersion = substr($version, 1,2);
        $patchVersion   = substr($version, 3,3);
        $codProdGrup    = $this->patchverprodRepository->getProdGrup($mainVersion, $releaseVersion, $patchVersion);
        
        $maxScriptStructure = $this->bkgthreadparamRepository->getMaxScriptStructure();

        if (empty($codProdGrup)){
            throw new CustomException('ERR_VERSION_NOT_REGISTERED');
        }

        $codArvProd = "P".$mainVersion;
        $productMaxLevel = PictureProduto::retornaNumeroMaximoDeNiveis($orgId)-1;
        $productsAvailables = $this->patchverprodRepository->getProductsAvailables($codArvProd, $productMaxLevel);
        
        if (empty($productsAvailables)){
            throw new CustomException('ERR_NONE_PRODUCTS_FOUND');
        }

        foreach ($productsAvailables as $product) {
            $patchVersionProduct = $this->patchverprodRepository->getPatchVersionProduct($clientId, $product['CDPRODUTO']);

            if ($patchVersionProduct){
                $productVersion = str_replace(".", "", $patchVersionProduct);

                if ($versionPatch == 'N' && $version < $productVersion){
                    $cdsProdDisp[] = $this->generateAvailablesProductsList($product, $patchVersionProduct, $languageId, $maxScriptStructure);
                } else {
                    $arvProd        = $product['CDARVPROD'];
                    $arvProd        = substr($arvProd, 1, 3);
                    $productVersion = substr($productVersion, 0, 3);
                    $version        = substr($version, 0, 3);

                    if ($version == $productVersion && $version == $arvProd){
                        $cdsProdDisp[] = $this->generateAvailablesProductsList($product, $patchVersionProduct, $languageId,$maxScriptStructure);
                    }
                }
            }
        }
        return $cdsProdDisp;
    }
    
    
   
    
    private function generateAvailablesProductsList($product, $patchVersionProduct, $languageId,$maxScriptStructure)
    {
        $auxProdDisp = array();
        $auxProdDisp['NMPRODUTO'] = $product['NMPRODUTO'];
        if($languageId == 'ESP' && !empty($product['NMPRODESPANH'])){
            $auxProdDisp['NMPRODUTO'] = $product['NMPRODESPANH'];
        } else if($languageId == 'ING' && !empty($product['NMPRODINGLES'])){
            $auxProdDisp['NMPRODUTO'] = $product['NMPRODINGLES'];
        }
        $auxProdDisp['CDPRODUTO'] = $product['CDPRODUTO'];
        $auxProdDisp['VERSAO']    = $patchVersionProduct;
        $auxProdDisp['VERSION_DELPHI_MIN'] = Utilities::getVersionDelphiMin();
        $auxProdDisp['MAX_SCRIPT_STRUCTURE'] = $maxScriptStructure;
        return $auxProdDisp;
    }
    
    private function validateAndFormatVersion($version)
    {
        $versionLength = strlen($version);
        $hasDots       = strpos($version,".");
        if ($versionLength === 8 && $hasDots !== false){
            $version = str_replace(".", "", $version);
        } else if ($versionLength != 5 || ( $versionLength === 5 && $hasDots === true)) {
            throw new CustomException('ERR_NOT_VALID_FORMAT_VERSION', 500, array( 'version' => $version));
        }
        
        return $version;
    }
    
    private function applyMaskVersion($version)
    {
        $maskaredVersion = '';
        if (strlen($version) === 6){
            $maskaredVersion  = substr($version, 0, 1) .".";
            $maskaredVersion .= substr($version, 1, 2) .".";
            $maskaredVersion .= substr($version, 3, 3);
        }
        return empty($maskaredVersion) ? $version : $maskaredVersion;
    }
    
    private function getUpdateType($requestType)
    {
        return isset($this->ALL_REQUESTS[$requestType]) ? $this->ALL_REQUESTS[$requestType]['DESCRIPTION']['REQUEST'] : "";
    }
    
    private function generateConditions($params)
    {
        $conditions = "";
        foreach($params as $key => $value){
           $conditions .= "AND EXISTS (SELECT BTP.IDTHREAD
                      FROM BKGTHREADPARAM BTP
                     WHERE BTP.IDTHREAD = BKG.IDTHREAD
                       AND BTP.DSPARTHREAD = '$key'
                       AND GET_BKG_DSVALPTHREAD(ROWID) = '$value')";
        }
        
        return $conditions;
    }
    
    private function getThreadNotConcluded($requestType, $clientId, $email, $dbId, $version, $productId, $languageId, $rdmMethod, $requestNumber, $patchLanguage, $params)
    {
        $conditions = $this->generateConditions($params);
        return $this->bkgthreadRepository->getThreadNotConcluded($clientId, $email, $dbId, $version, $productId, $languageId, $rdmMethod, $conditions);
    }
    
    private function buildMessageWhenHasRequestThread($requestType, $product, $dbId, $version, $updateType, $thread)
    {
        $msg = array(
            'CODE'   => 'ERR_DOWNLOAD_VERSION_ALREADY_REQUESTED',
            'PARAMS' => array(
                'product'        => $product->getNmproduto(),
                'dbId'           => $dbId,
                'currentVersion' => $version ? $version : '-',
                'updateType'     => $updateType,
                'requestData'    => $thread['DTINCLTHREAD'],
            )
        );
            
        if ($requestType == "GENERATE_PATCH"){
            $msg['PARAMS']['type'] = 'patch';
        } else if ($requestType == "GENERATE_FIX"){
            $msg['PARAMS']['type'] = 'fix';
        } else if ($requestType == "GENERATE_LANGUAGE_PATCH"){
            $msg['PARAMS']['type'] = 'patch de idioma';
        } else if ($requestType == "GENERATE_SQL_PATCH"){
            $msg['PARAMS']['type'] = 'patch de SQL';
        }

        if ($thread['DTEXECTHREAD']){
            $msg['PARAMS']['executionData'] = 'INÍCIO DA EXECUÇÃO DA SOLICITAÇÃO: ' . $thread['DTEXECTHREAD'];
        } else {
            $msg['PARAMS']['executionData'] = '';
        }
        $msg['PARAMS']['type1'] =$msg['PARAMS']['type'];
        return $msg;
    }
    
    private function checkHasRequestThread($requestType, $clientId, $email, $dbId, $version, $productId, $languageId, $rdmMethod, $requestNumber, $patchLanguage, $params)
    {
        $msg = false;
        $thread = $this->getThreadNotConcluded($requestType, $clientId, $email, $dbId, $version, $productId, $languageId, $rdmMethod, $requestNumber, $patchLanguage, $params);
        if ($thread){
            if ($thread['DTINCLTHREAD'] < (new \DateTime())->modify('-1 day')){
                $product = $this->produtoRepository->findOneBy(array('cdproduto' => $productId));
                $updateType = $this->getUpdateType($requestType);
                $msg = $this->buildMessageWhenHasRequestThread($requestType, $product, $dbId, $version, $updateType, $thread);
            } else {
                $this->changeThreadStatus($thread['IDTHREAD'], Bkgthread::SOAP_RESPONSE_ERROR_ID, Messages::ERR_UNKNOWN_ON_REQUEST_PROCESSING);
            }
        }
        
        return $msg;
    }
    
    public function changeThreadStatus($threadId, $statusId, $errorMessage = null)
    {
        try {
            $dateNow   = new \DateTime();
            $bkgthread = $this->bkgthreadRepository->findOneBy(array('idthread' => $threadId));
            $bkgthread->setIdstatthread($statusId);
            if ($statusId == Bkgthread::REQUEST_IN_EXECUTION_ID) {
                $bkgthread->setDtexecthread($dateNow);
            } else  if ($statusId == Bkgthread::SOAP_RESPONSE_SUCCESS_ID) {
                $bkgthread->setDttermthread($dateNow);
            } else  if ($statusId == Bkgthread::SOAP_RESPONSE_ERROR_ID) {
                $bkgthread->setDttermthread($dateNow);
                $bkgthread->setDserrothread($errorMessage);
            }
            
            $this->entityManager->persist($bkgthread);
            $this->entityManager->flush();
        } catch (\Exception $ex){
            throw new \Exception($ex->getMessage());
        }
    }
    
    private function getRdmMethod($requestType)
    {
        return isset($this->ALL_REQUESTS[$requestType]) ? $this->ALL_REQUESTS[$requestType]['RDM_METHOD'] : "";
    }
    
    private function generateParams($requestType, $clientId, $productId, $version, $email, $dbId, $languageId, $requestNumber, $initialPatch, $lastPatch, $patchLanguage, $mode)
    {

        $params = array(
            "000" => $clientId,
            "001" => $productId,
            "002" => $version,
            "003" => $email,
            "004" => $dbId,
            "005" => $languageId
        );
        
        if ($requestType == "GENERATE_FIX"){
            $params["006"] = $requestNumber;
        } else if ($requestType == "GENERATE_LANGUAGE_PATCH"){
            $params["002"] = $initialPatch;
            $params["004"] = $languageId;
            $params["005"] = $patchLanguage;
            $params["006"] = $lastPatch;
            $params["007"] = $mode;
        } else if ($requestType == "GENERATE_SQL_PATCH"){
            $params["002"] = $initialPatch;
            $params["003"] = $lastPatch;
            $params["004"] = $email;
            $params["005"] = $dbId;
            $params["006"] = $languageId;
            $params["007"] = $mode;
        }
        
        return $params;
    }
    
    private function getRequestTypeByRdm($rdmMethod)
    {
        $requestType = null;
        $positionRequest = array_search($rdmMethod, array_column($this->ALL_REQUESTS, 'RDM_METHOD'));
        if ($positionRequest !== false){
            $newAllRequests = array_keys($this->ALL_REQUESTS);
            $requestType = $newAllRequests[$positionRequest];
        }
        return $requestType;
    }
    
    private function showInLogRequestData($thread)
    {
        $description = array(
            "GENERATE_PATCH"            => array ("Customer ID: ", "Product ID: ", "Current version: ", "E-mail: ", "Database ID: ", "Language ID: "),
            "GENERATE_FIX"              => array ("Customer ID: ", "Product ID: ", "Current version: ", "E-mail: ", "Database ID: ", "Language ID: ", "Request number: "),
            "GENERATE_LANGUAGE_PATCH"   => array ("Customer ID: ", "Product ID: ", "Initial patch: ", "E-mail: ", "Language ID: ", "Patch language: ", "Last patch: ", "Mode: "),
            "GENERATE_SQL_PATCH"        => array ("Customer ID: ", "Product ID: ", "Initial patch: ", "Last patch: ", "E-mail: ", "Database ID: ", "Language ID: ", "Mode: ")
        );

        $requestType     = $this->getRequestTypeByRdm($thread->getDsnobjthread());
        $bkgthreadparams = $this->bkgthreadparamRepository->getDsvalpthreadByThreadId($thread->getIdthread());
        $currentDateTime = "\n[". (new \DateTime())->format('d/m/Y H:i:s') ."] ";

        foreach ($bkgthreadparams as $key => $param) {
            echo $currentDateTime . $description[$requestType][$key] . $param['DSVALPTHREAD'];
        }
       
        echo "\n";
    }
    
    public function getProductVersionsByPatchesInterval($initialPatch, $lastPatch)
    {
        $initialPatch = $this->validateAndFormatVersion($initialPatch);
        $lastPatch    = $this->validateAndFormatVersion($lastPatch);
        return $this->solipatchRepository->getProductVersionsByPatchesInterval($initialPatch, $lastPatch);
    }
    
    private function generateDescriptionToThread($requestType, $customerName, $requestNumber)
    {
        $description = isset($this->ALL_REQUESTS[$requestType]) ? $this->ALL_REQUESTS[$requestType]['DESCRIPTION']['GENERATION'] : "";
        return str_replace(array('{{customerName}}', '{{requestNumber}}'), array($customerName, $requestNumber), $description);
    }

    public function requestDownloadVersion($requestType, $dbId, $email, $version, $initialPatch, $lastPatch,
                                           $productId, $languageId, $requestNumber, $patchLanguage, $mode,
                                           $versionStructure = null)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();
            if($requestType == "GENERATE_LANGUAGE_PATCH" || $requestType == "GENERATE_SQL_PATCH"){
                if ($requestType == "GENERATE_SQL_PATCH"){
                    $isAuthorized = $this->getAuthorizationByCurrentUser();
                    if (!$isAuthorized){
                        throw new CustomException('ERR_OPERATOR_NOT_AUTHORIZED');
                    }
                }
                $initialPatch = $this->validateAndFormatVersion($initialPatch);
                $lastPatch    = $this->validateAndFormatVersion($lastPatch);
            } else {
                if ($requestType == "GENERATE_FIX"){
                    $request = $this->soliservRepository->findOneBy(array('nrsoliserv' => $requestNumber));
                    if(empty($request)){
                        throw new CustomException('ERR_REQUEST_NOT_REGISTERED');
                    }
                }
                $version = $this->validateAndFormatVersion($version);
            }
            $userId     = $this->environment->getCurrentUserId();
            $clientId   = $this->environment->getCurrentCdCliente();
            $orgId      = $this->environment->getCurrentOrganizationId();
            $client     = $this->clienteRepository->findOneBy(array('cdcliente' => $clientId));
            $rdmMethod  = $this->getRdmMethod($requestType);
            if ($client){
                $versionDelphiStructure = Utilities::getVersionDelphiMin();
                if($versionDelphiStructure && !is_null($versionStructure) && $version){
                    $versionMin = Utilities::onlyNumber($versionDelphiStructure);
                    $versionCurrent = Utilities::onlyNumber($version);
			        if($versionStructure === "0" || $versionStructure === 0){
                        $versionStructure = Utilities::getDefaultDataBaseVersion();
                    }
                    if($versionCurrent >= $versionMin){
                        $dbId = $dbId . '=' . $versionStructure;
                    }
                }
                $params = $this->generateParams($requestType, $clientId, $productId, $version, $email, $dbId, $languageId, $requestNumber, $initialPatch, $lastPatch, $patchLanguage, $mode);
                $hasRequestThread = $this->checkHasRequestThread($requestType, $clientId, $email, $dbId, $version, $productId, $languageId, $rdmMethod, $requestNumber, $patchLanguage, $params);
            
                if ($hasRequestThread){
                    throw new CustomException($hasRequestThread['CODE'], 500, $hasRequestThread['PARAMS']);
                }
                
                $this->updateClientDb($clientId, $dbId);
                
                $description     = $this->generateDescriptionToThread($requestType, $client->getNmrazsocclie(), $requestNumber);
                $bkgthreadEntity = $this->bkgthreadFactory->create($userId, $orgId, $description, $rdmMethod);
                
                $this->entityManager->persist($bkgthreadEntity);
                $this->entityManager->flush();
                $this->downloadVersionProducer->sendRequest($bkgthreadEntity->getIdthread());
                $this->saveThreadParam($params, $bkgthreadEntity->getIdthread(), $orgId, $userId);
                
                $connection->commit();
            } else {
                throw new CustomException('ERR_NOT_FOUND_CUSTOMER');
            }
            
        } catch (CustomException $ex){
            $connection->rollBack();
            throw $ex;
        }
    }
    
    private function saveThreadParam($params, $threadId, $orgId, $userId)
    {
        foreach($params as  $threadParamId => $threadValue){
            $threadTypeId = $threadParamId == Bkgthreadparam::THREAD_PARAM_ID ? Bkgthreadparam::THREAD_TYPE_RDM_METHOD_ID : Bkgthreadparam::THREAD_TYPE_PARAMS_ID;
            $bkgthreadparamEntity = $this->bkgthreadparamFactory->create($threadId, $orgId, $userId, $threadParamId, $threadTypeId, $threadValue);
            $this->entityManager->persist($bkgthreadparamEntity);
            $this->entityManager->flush();
        }
    }
    
    private function getArgsSoap($rdmMethod, $threadParams,$threadId = null)
    {
        $initValue = '1';
        if($threadId){
            $initValue = $threadId;
        }
        $args = array(
            new \SoapVar($initValue, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
        );
    
        foreach($threadParams as $param){
            $index = intval($param->getDsparthread())+1;
            $args[$index] = new \SoapVar(stream_get_contents($param->getDsvalpthread()), XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema");
        }
        
        return $args;
    }
    
    public function downloadVersion($threadId)
    {
        try {
            ini_set('default_socket_timeout', 7200);
            
            $thread         = $this->bkgthreadRepository->findOneBy(array('idthread' => $threadId));
            $threadParams   = $this->bkgthreadparamRepository->findBy(array('idthread' => $threadId));
            $rdmMethod      = $thread->getDsnobjthread();
            $arrayRdmMethod = explode('.', $rdmMethod);
            $args           = $this->getArgsSoap($rdmMethod, $threadParams,$threadId);

            $this->changeThreadStatus($threadId, Bkgthread::REQUEST_IN_EXECUTION_ID);
            $this->showInLogRequestData($thread);
            $url = Utilities::getUrlWSDLPatchTeknisa();
            
            $options = Utilities::getOptionsSoap();
    
            $params = array(
                'rdm'    => $arrayRdmMethod[0] .".".  $arrayRdmMethod[1],
                'method' => $arrayRdmMethod[2],
                'args'   => $args
            );

            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            return $this->checkResponseSoap($result, $threadId);
        } catch (\SoapFault $ex) {
            return $this->checkResponseSoap(null, $threadId, $ex);
        }
    }
    
    private function checkResponseSoap($result, $threadId, $ex = null)
    {
        if (empty($result)  && !$ex){
            $responseSoap = Messages::ERR_REQUEST_DOWNLOAD_VERSION_UNKNOWN."unknown";
        } else if (!empty($result) && isset($result->executeCOMMethodResult)){
            if ($result->executeCOMMethodResult == self::SOAP_RESPONSE_SUCCESS) {
                $this->changeThreadStatus($threadId, Bkgthread::SOAP_RESPONSE_SUCCESS_ID);
                $responseSoap = Messages::SUC_REQUEST_PATCH_DOWNLOAD_VERSION;
            } else if (substr($result->executeCOMMethodResult, 0,5) == self::SOAP_INITIAL_RESPONSE_ERROR) {
                $messageError = substr($result->executeCOMMethodResult, 5,strlen($result->executeCOMMethodResult));
                $this->changeThreadStatus($threadId, Bkgthread::SOAP_RESPONSE_ERROR_ID, $messageError);
                $responseSoap = Messages::ERR_REQUEST_DOWNLOAD_VERSION_WITH_DETAILS . $messageError;
            } else {
                $this->changeThreadStatus($threadId, Bkgthread::SOAP_RESPONSE_ERROR_ID, $result->executeCOMMethodResult);
                $responseSoap = Messages::ERR_REQUEST_DOWNLOAD_VERSION_WITH_DETAILS . $result->executeCOMMethodResult;
            }
        } else if (!empty($ex)){
            $this->changeThreadStatus($threadId, Bkgthread::SOAP_RESPONSE_ERROR_ID, $ex->getMessage());
            $responseSoap = Messages::ERR_REQUEST_DOWNLOAD_VERSION_WITH_DETAILS . $ex->getMessage();
        }  else {
            $this->changeThreadStatus($threadId, Bkgthread::SOAP_RESPONSE_ERROR_ID, Messages::ERR_UNEXPECTED_REQUEST_DOWNLOAD_VERSION);
            $responseSoap = Messages::ERR_REQUEST_DOWNLOAD_VERSION_WITH_DETAILS . Messages::ERR_UNEXPECTED_REQUEST_DOWNLOAD_VERSION;
        }
        
        return $responseSoap;
    }
    
    private function generateHTMLTable($itemsTask, $initialPatch, $endPatch)
    {
        $initialPatch   = $this->applyMaskVersion($initialPatch);
        $endPatch       = $this->applyMaskVersion($endPatch);
        $totalItemsTask = count($itemsTask);
        $html = "<!DOCTYPE html>
        <html>
        <head>
        <meta http-equiv='Content-Type' content='text/html; charset=utf-8'/>
        <style>
            body {
                font-family: arial;
            	font-size: 110%;
            }
            div {
            }
            table {
            	width: 100%;
            	border: 0px;
            	border-collapse: collapse;
            }
            td {
            	border: 0px;
            }
            
            .texto10 {
                text-decoration: none;
                font-weight: bold;
                font-family: Tahoma;
                font-size: 10pt;
            }
            .txt-grid {
                border: none;
                font-family: Tahoma;
                font-size: 8pt;
                background-color: transparent;
            }.texto9 {
                text-decoration: none;
                font-family: Tahoma;
                font-size: 9pt;
                font-weight: bold;
            }
            .texto8 {
                text-decoration: none;
                font-weight: bold;
                font-family: Tahoma;
                font-size: 8pt;
            }
            .bodyBranco {
                background-color: #ffffff;
            }
            </style>
            </head>
            <body class='bodyBranco'>
            <br/>
        <table cellpadding='0' cellspacing='0' style='width:100%;border: 0px;'>
            <tr>
               <td colspan='2' class='texto10' align='center'>&nbsp;Arquivos Alterados</td>
            </tr>
            <tr>
               <td colspan='2' align='center' style='height:20'>
                  <div style='height:09; clip:auto; overflow:hidden; width:100%'>
                     <hr style='color:#000000'/>
                  </div>
               </td>
            </tr>
            <tr>
               <td colspan='1' class='texto9' style='width:20%'>Intervalo da Patch:</td>
               <td colspan='1' style='text-decoration: none;font-family: Tahoma;font-size: 9pt;'> De ". $initialPatch ." até ". $endPatch ."</td>
            </tr>
            <tr>
               <td colspan='2' align='center'>
                  <div style='height:9px;  width:100%'> 
                     <hr style='color:#000000'/>
                  </div>
               </td>
            </tr>
        </table>
        <div>    
         <table cellpadding='0' cellspacing='0' style='border:0px;width:100%'>
         <tbody>
            <tr>
               <td class='txt-grid' style='width:12%'></td>
               <td class='txt-grid' style='width:40%'></td>
               <td class='txt-grid' style='width:40%'></td>
               <td class='txt-grid'></td>
               <td class='txt-grid'></td>
            </tr>";
            
        $lastRequestNumber = null;
        $lastTask = null;
        $lastCodItemWork = null;

        if ($totalItemsTask > 0){
            foreach($itemsTask as $request){
                if (!strlen($request['DSSOLISERV']) || substr($request['DSSOLISERV'],0,5) != '{\rtf') {
                    $description = utf8_encode($request['DSSOLISERV']);
                } else {
                    $description = Utilities::rtf2textPlain(trim($request['DSSOLISERV']), true);
                }
                $lastRequestNumber = $request['NRSOLISERV'];
                $html .= "<tr height='20px'>";
                $html .= "   <td colspan='5'><hr style='color:#000000'/></td>";
                $html .= "</tr>";
                $html .= "<tr height='20px'>";
                $html .= "   <td class='txt-grid texto8' colspan='1'>Solicitação:                              </td>";
                $html .= "   <td class='txt-grid texto8' colspan='4'>             ". $request['NRSOLISERV']     ."</td>";
                $html .= "</tr>";         
                $html .= "<tr height='20px'>";
                $html .= "   <td class='txt-grid; texto8' colspan='1' valign='top'>Descrição:                   </td>";
                $html .= "   <td class='txt-grid' colspan='4'> ". nl2br($description)  ."</td>";
                $html .= "</tr>";
                
                foreach ($request['TAREFAS'] as $tarefa) {
                    $html .= "<tr><td colspan='5' style='height:10px;'></td></tr>";
                    $html .= "<tr height='20px'>";
                    $html .= "   <td class='txt-grid' colspan='1'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;              </td>";
                    $html .= "   <td class='txt-grid texto8' colspan='4'>Tarefa:      ". $tarefa['NRTAREFA']       ."</td>";
                    $html .= "</tr>";
                    $html .= "<tr height='20px'>";
                    $html .= "   <td class='txt-grid' colspan='1'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;              </td>";
                    $html .= "   <td class='txt-grid texto8' colspan='1'>Código Arquivo.                           </td>";
                    $html .= "   <td class='txt-grid texto8' colspan='3'>Nome Arquivo.                             </td>";
                    $html .= "</tr>";
                    foreach ($tarefa['ITEMS'] as $item) {
                        if(isset($item['ARQUIVOS'])){
                            foreach ($item['ARQUIVOS'] as $arquivo) {
                                $html .= "<tr>";
                                $html .= "   <td class='txt-grid' colspan='1'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;              </td>";
                                $html .= "   <td class='txt-grid' colspan='1'>                     " . $arquivo['CDREDUITTRAB']  ."</td>";
                                $html .= "   <td class='txt-grid' colspan='3'>                     ". $arquivo['NMITEMTRAB']     ."</td>";
                                $html .= "</tr>";
                            }
                        }
                    }
                }
            }
            $html .= "</tr>";
        }
        $html .=  "<tr height='20px'><td class='txt-grid' colspan='5'></td></tr></tbody></table></div></body></html>";
        return $html;
    }
    
    private function generateQueryTasks($patchRequests)
    {
        $totalPatchRequests = count($patchRequests);
        $auxTotalPatchRequests = 0;
        $query = "SELECT NRTAREFA, NRSOLISERV, CDCLIENTE FROM TAREFA WHERE (";

        foreach($patchRequests as $index => $request){
            if ($index == 0) {
                $query .= ' NRSOLISERV = '. $request['NRSOLISERV'];
            } else if ($auxTotalPatchRequests < $totalPatchRequests){
                $query .= ' OR NRSOLISERV = '. $request['NRSOLISERV'];
            } 
            $auxTotalPatchRequests++;
        } 
        $query .= ') ORDER BY NRSOLISERV, NRTAREFA';
        
        return $query;
    }
    
    private function generateQueryTaskItems($tasks)
    {
        $totalTasks = count($tasks);
        $auxTotalTasks = 0;
        $query = "
        SELECT TSI.NRSOLISERV, TSI.NRTAREFA, TSI.CDITEMTRAB, TSI.NMITEMTRAB, TSI.CDREDUITTRAB, DES.DSSOLISERV 
          FROM 
            (SELECT  T.NRSOLISERV, I.NRTAREFA, I.CDITEMTRAB, IT.NMITEMTRAB, IT.CDREDUITTRAB 
               FROM ITEMTAREFA I
               JOIN ITEMTRAB IT ON I.CDITEMTRAB = IT.CDITEMTRAB
               JOIN TAREFA T ON I.NRTAREFA = T.NRTAREFA 
               JOIN DIRFONTITEMTRAB D ON IT.CDITEMTRAB = D.CDITEMTRAB
              WHERE I.NRTAREFA IN (";
        foreach($tasks as $index => $task){
            if ($index == 0) {
                $query .= $task['NRTAREFA'];
            } else if ($auxTotalTasks < $totalTasks){
                $query .= " , ". $task['NRTAREFA'];
            } 
            $auxTotalTasks++;
        } 
        $query .=  ") GROUP BY T.NRSOLISERV, I.NRTAREFA, I.CDITEMTRAB, IT.NMITEMTRAB, IT.CDREDUITTRAB 
        ORDER BY I.NRTAREFA, I.CDITEMTRAB) TSI
        JOIN DESCSOLI DES ON TSI.NRSOLISERV = DES.NRSOLISERV";
        
        return $query;
    }

    public function getReportChangedFiles($initialPatch, $endPatch)
    {
        ini_set('max_execution_time', 300);
        set_time_limit(300);
        try {
            $initialPatch = $this->validateAndFormatVersion($initialPatch);
            $endPatch     = $this->validateAndFormatVersion($endPatch);
            $itemsTask    = $this->solipatchRepository->getItemsTask($initialPatch, $endPatch);
            $itemsTask    = $this->handleItemsTask($itemsTask);
            return $this->generateHTMLTable($itemsTask, $initialPatch, $endPatch);
        } catch (\Exception $ex){
            throw new \Exception($ex->getMessage());
        }
    }
    
    private function handleItemsTask($itemsTask){
        $items = array();
        foreach($itemsTask as $item){
            $indexSolicitacao = array_search($item['NRSOLISERV'], array_column($items, 'NRSOLISERV'));
            if($indexSolicitacao > -1){
                $indexTarefa = array_search($item['NRTAREFA'], array_column($items[$indexSolicitacao]['TAREFAS'], 'NRTAREFA'));
                if($indexTarefa > -1){
                    $indexItem = array_search($item['CDITEMTRAB'], array_column($items[$indexSolicitacao]['TAREFAS'][$indexTarefa]['ITEMS'], 'CDITEMTRAB'));
                    if($indexItem > -1){
                        $items[$indexSolicitacao]['TAREFAS'][$indexTarefa]['ITEMS'][$indexItem]['ARQUIVOS'][] = array(
                            'CDREDUITTRAB' => $item['CDREDUITTRAB'],
                            'NMITEMTRAB' => $item['NMITEMTRAB']
                        );
                    } else {
                        $items[$indexSolicitacao]['TAREFAS'][$indexTarefa]['ITEMS'][] = array(
                            'CDITEMTRAB' => $item['CDITEMTRAB'],
                            'ARQUIVOS' => array(
                                array(
                                    'CDREDUITTRAB' => $item['CDREDUITTRAB'],
                                    'NMITEMTRAB' => $item['NMITEMTRAB']
                                )
                            )
                        );
                    }
                } else {
                    $items[$indexSolicitacao]['TAREFAS'][] = array(
                        'NRTAREFA' => $item['NRTAREFA'],
                        'ITEMS' => array(
                            array(
                                'NMITEMTRAB' => $item['NMITEMTRAB'],
                                'ARQUIVOS' => array(
                                    array(
                                        'CDREDUITTRAB' => $item['CDREDUITTRAB'],
                                        'NMITEMTRAB' => $item['NMITEMTRAB']
                                    )
                                )
                            )
                        )
                    );
                }
            } else {
                $items[] = array(
                    'NRSOLISERV' => $item['NRSOLISERV'],
                    'DSSOLISERV' => trim($item['DSSOLISERV']),
                    'TAREFAS' => array(
                        array(
                            'NRTAREFA' => $item['NRTAREFA'],
                            'ITEMS' => array(
                                array(
                                    'CDITEMTRAB' => $item['CDITEMTRAB'],
                                    'ARQUIVOS' => array(
                                        array(
                                            'CDREDUITTRAB' => $item['CDREDUITTRAB'],
                                            'NMITEMTRAB' => $item['NMITEMTRAB']
                                        )
                                    )
                                )
                            )
                        )
                    )
                );
            }
        }
        
        return $items;
    }
}
