# Zeedhi Layout Preferences

## Introduction
Provides the methods to change how data is displayed on Zeedhi Grid widgets and reports and save these layouts.

**zh-layout-preferences** provides the following features:

 - Allow the user to customize Zeedhi Grids
 - Save layouts for the users
 - Allow the user to define a default layout for a widget

## Requirements

This project requires:

 - **[Zeedhi Preferences][ZH_PREFERENCES_FRONTEND_API]** v3.0.0 or higher
 - A backend project that uses **[zh-preferences][ZH_PREFERENCES_BACKEND_API]** backend library

## How to start
In order to start using the layout preferences you need to add the *JS* file into the `index.html` file of your project.
The files are located inside your project's **bower_components** folder in `zh-layout-preferences/dist` directory. The zeedhi-component zh-grid-configuration come as bower dependencies, and you should include its css and main.js as well.

**JS:**
```html
<!-- Preferences scripts -->
<script src="bower_components/zh-layout-preferences/dist/layout-preferences.min.js" />
<script src="bower_components/zh-grid-configuration/dist/zh-grid-configuration.min.js"></script>

<!-- zh-grid-configuration stylesheets -->
<link rel="stylesheet" href="bower_components/zh-grid-configuration/dist/zh-grid-configuration.css">
```
You can also use the non minified version by changing the file name to **layout-preferences.js**.

## Configuration
**Zh Layout Preferences** require some package configuration.

You need to define the **templateUrl** package that will be used to retrieve the templates used by layout preferences.
**E.g.:**
```
window.templateUrls.push({
    package: 'zh-layout-preferences',
    baseUrl: 'bower_components/zh-layout-preferences/dist/templates/'
    });
```

You also need to define the **metadataUrl** package that will be used to retrieve the datasources and configuration widgets files.
**E.g.:**
```
window.metadataUrls.push({
    package: 'zh-layout-preferences',
    baseUrl: 'bower_components/zh-layout-preferences/dist/assets/'
});
```

And to define the **serviceUrl** that will be used to retrieve the layouts.
**E.g.:**
```
window.serviceUrls.push({
    package: 'zh-layout-preferences',
    baseUrl: '../backend/service/index.php'
});
```
The path used on the **serviceUrl** must be a Zeedhi Project that uses the **zh-preferences** backend library. More info can be found [**here**][ZH_PREFERENCES_BACKEND_API].

> The library already register it's packages using the **default** configuration shown on the examples above.
>
> If you need to override any configuration you must add the package into the **window** variable before adding the JS layout preferences file.

**Note** that all packages must be named **zh-layout-preferences**.

## Usage

### Remove fields from customization

The widget used to create a **layout** allows all fields from the customized widget to be edited.

The property **skipLayout** allows you to remove a field from the customization options, setting it to **true** will prevent it from being displayed on the options grid.

### Enable layout preferences

To enable **zh-layout-preferences** you have to:

 - Set the value of the property **floatingControl.customizationAction.layout** to **true**.
 - Set a **unique ID** in the widget
 - Set the authenticated user on **[ZhPreferences][ZH_PREFERENCES_FRONTEND_API]**.

## Layout events
There are two events that are triggered when handling layouts: **LayoutBeforeApply** and **LayoutAfterApply**.

**LayoutBeforeApply** is triggered before the widget is updated with the changes made by the user.

The callback listener receives an object containing *Zeedhi* basic event data, and a **e** property that is an object with the property **layout**.

**layout** is a *Layout* contaning the preference the user is applying.

**LayoutAfterApply** is triggered after the widget is updated with the changes made by the user.

The callback listener receives an object containing *Zeedhi* basic event data, and a **e** property that is an object with the property **layout**.

**layout** is a *Layout* contaning the preference the user is applying.

## Samples

You can see how to configure and use in [Zeedhi Preferences Samples][ZEEDHI_PREFERENCES_SAMPLES].


## All related projects
 - [Zeedhi Preferences Samples][ZEEDHI_PREFERENCES_SAMPLES]
 - [Zh Preferences Backend API][ZH_PREFERENCES_BACKEND_API]
 - [Zh Preferences Frontend API][ZH_PREFERENCES_FRONTEND_API]
 - [Zh Filter Preferences][ZH_FILTER_PREFERENCES]
 - [Zh Layout Preferences][ZH_LAYOUT_PREFERENCES]
 - [Zh View Preferences][ZH_VIEW_PREFERENCES]

## Advanced Usage
For advanced usage you can visit the **[API documentation][DOC]**.

[DOC]: http://docs.zeedhi.com/docs/preferences/layout-preferences
[ZEEDHI_PREFERENCES_SAMPLES]: http://code.zeedhi.com/zeedhi/zeedhi-preferences-samples/tree/master
[ZH_PREFERENCES_BACKEND_API]: http://code.zeedhi.com/zeedhi/zh-preference-backend/tree/master
[ZH_PREFERENCES_FRONTEND_API]: http://code.zeedhi.com/zeedhi-components/zh-customizations-service/tree/master
[ZH_FILTER_PREFERENCES]: http://code.zeedhi.com/zeedhi-components/zh-filter-preferences/tree/master
[ZH_LAYOUT_PREFERENCES]: http://code.zeedhi.com/zeedhi-components/zh-layout-preferences/tree/master
[ZH_VIEW_PREFERENCES]: http://code.zeedhi.com/zeedhi-components/zh-customization-preferences/tree/maste